﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.PDPSimulation.Operators;
using HeuristicLab.Problems.VehicleRouting;
using HeuristicLab.Problems.VehicleRouting.Interfaces;
using HeuristicLab.Parameters;
using HeuristicLab.Data;

namespace HeuristicLab.PDPSimulation {
  [Item("DepotWaitingStrategy", @"A pickup and delivery waiting strategy. It has been implemented as described in: 
Branke, Jürgen, et al. Waiting strategies for dynamic vehicle routing. Transportation science 39.3 (2005): 298-312.")]
  [StorableClass]
  public class DepotWaitingStrategy : WaitingStrategy {
    public DepotWaitingStrategy()
      : base() {
    }
    [StorableConstructor]
    protected DepotWaitingStrategy(bool deserializing) : base(deserializing) { }
    protected DepotWaitingStrategy(DepotWaitingStrategy original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new DepotWaitingStrategy(this, cloner);
    }

    public override double GetWaitingTime(double time, DynPDPProblemInstance instance, IVRPEncoding solution, Tour tour, int stop) {
      int vehicle = solution.GetVehicleAssignment(solution.GetTourIndex(tour));
      if (stop == 0 && 
        instance.Coordinates[vehicle, 0] == instance.DepotCoordinates[vehicle, 0] &&
        instance.Coordinates[vehicle, 1] == instance.DepotCoordinates[vehicle, 1]) {
          return WaitFirstWaitingStrategy.GetSlack(time, instance, solution, tour, stop);
      } else {
        return 0;
      }
    }
  }
}
