﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.PDPSimulation.Operators;
using HeuristicLab.Problems.VehicleRouting;
using HeuristicLab.Problems.VehicleRouting.Interfaces;
using HeuristicLab.Parameters;
using HeuristicLab.Data;

namespace HeuristicLab.PDPSimulation {
  [Item("DistWaitingStrategy", @"A pickup and delivery waiting strategy. It has been implemented as described in: 
Branke, Jürgen, et al. Waiting strategies for dynamic vehicle routing. Transportation science 39.3 (2005): 298-312.")]
  [StorableClass]
  public class DistWaitingStrategy : WaitingStrategy {
    public ValueParameter<BoolValue> VariableWaitingParameter {
      get { return (ValueParameter<BoolValue>)Parameters["VariableWaiting"]; }
    }

    public DistWaitingStrategy()
      : base() {
        Parameters.Add(new ValueParameter<BoolValue>("VariableWaiting", "Indicates, if variable waiting should be used.", new BoolValue(false)));
    }
    [StorableConstructor]
    protected DistWaitingStrategy(bool deserializing) : base(deserializing) { }
    protected DistWaitingStrategy(DistWaitingStrategy original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new DistWaitingStrategy(this, cloner);
    }

    private double GetDistance(DynPDPProblemInstance instance, IVRPEncoding solution, Tour tour, int stop, out double stopDistance) {
      stopDistance = 0;
      double distance = 0;

      for (int i = stop - 1; i < tour.Stops.Count; i++) {
        double dist = instance.GetDistance(i >= 0 ? tour.Stops[i] : 0,
                                           i + 1 < tour.Stops.Count ? tour.Stops[i + 1] : 0, solution);

        distance += dist;
        if (i == stop - 1)
          stopDistance = dist;
      }

      return distance;
    }


    public override double GetWaitingTime(double time, DynPDPProblemInstance instance, IVRPEncoding solution, Tour tour, int stop) {
      double slack = WaitFirstWaitingStrategy.GetSlack(time, instance, solution, tour, stop);

      double stopDistance;
      double distance = GetDistance(instance, solution, tour, stop, out stopDistance);

      if (!VariableWaitingParameter.Value.Value || distance <= slack)
        return slack * (distance == 0 ? 0 : stopDistance / distance);
      else
        return 0;
    }
  }
}
