﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.PDPSimulation.Operators;
using HeuristicLab.Problems.VehicleRouting;
using HeuristicLab.Problems.VehicleRouting.Interfaces;
using HeuristicLab.Parameters;
using HeuristicLab.Data;

namespace HeuristicLab.PDPSimulation {
  [Item("LocationWaitingStrategy", @"A pickup and delivery waiting strategy. It has been implemented as described in: 
Branke, Jürgen, et al. Waiting strategies for dynamic vehicle routing. Transportation science 39.3 (2005): 298-312.")]
  [StorableClass]
  public class LocationWaitingStrategy : WaitingStrategy {
    public LocationWaitingStrategy()
      : base() {
    }
    [StorableConstructor]
    protected LocationWaitingStrategy(bool deserializing) : base(deserializing) { }
    protected LocationWaitingStrategy(LocationWaitingStrategy original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new LocationWaitingStrategy(this, cloner);
    }

    public override double GetWaitingTime(double time, DynPDPProblemInstance instance, IVRPEncoding solution, Tour tour, int stop) {
      double slack = WaitFirstWaitingStrategy.GetSlack(time, instance, solution, tour, stop);

      return slack / (double)(tour.Stops.Count - stop);
    }
  }
}
