﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.PDPSimulation.Operators;
using HeuristicLab.Problems.VehicleRouting;
using HeuristicLab.Problems.VehicleRouting.Interfaces;
using HeuristicLab.Parameters;
using HeuristicLab.Data;

namespace HeuristicLab.PDPSimulation {
  [Item("MaxDistWaitingStrategy", @"A pickup and delivery waiting strategy. It has been implemented as described in: 
Branke, Jürgen, et al. Waiting strategies for dynamic vehicle routing. Transportation science 39.3 (2005): 298-312.")]
  [StorableClass]
  public class MaxDistWaitingStrategy : WaitingStrategy {
    public MaxDistWaitingStrategy()
      : base() {
    }
    [StorableConstructor]
    protected MaxDistWaitingStrategy(bool deserializing) : base(deserializing) { }
    protected MaxDistWaitingStrategy(MaxDistWaitingStrategy original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MaxDistWaitingStrategy(this, cloner);
    }

    private int GetMaxDistStop(DynPDPProblemInstance instance, IVRPEncoding solution, Tour tour) {
      int maxDistStop = 0;
      double maxDist = double.MinValue;

      int vehicle = solution.GetVehicleAssignment(solution.GetTourIndex(tour));

      for (int i = 0; i < tour.Stops.Count; i++) {
        double distance = instance.Scenario.DistanceMeasure.GetDistance(
          instance.DepotCoordinates[vehicle, 0], instance.DepotCoordinates[vehicle, 1], 
          instance.GetCoordinates(tour.Stops[i])[0], instance.GetCoordinates(tour.Stops[i])[1]);
        if (distance > maxDist) {
          maxDist = distance;
          maxDistStop = i;
        }
      }

        return maxDistStop;
    }

    public override double GetWaitingTime(double time, DynPDPProblemInstance instance, IVRPEncoding solution, Tour tour, int stop) {
      if (stop == GetMaxDistStop(instance, solution, tour)) {
        return WaitFirstWaitingStrategy.GetSlack(time, instance, solution, tour, stop);
      } else {
        return 0;
      }
    }
  }
}
